require "spec_helper"
require 'active_record'
require "ohm"
Dir[File.dirname(__FILE__) + '../lib/weather-model.rb'].each { |file| require file }

# include WeatherModel

RSpec.describe WeatherModel do

  before do
    # start an ohm
    WeatherModel.set_ohm Redic.new("redis://127.0.0.1:6379/5")
    # run migration
    Dir[File.dirname(__FILE__) + '/../db/migrate/*.rb'].each { |file| require  file }
    # drop table
    FileUtils.rm_f 'weather-model-test.sqlite3'
    ActiveRecord::Base.establish_connection(
      :adapter => 'sqlite3',
      :database => 'weather-model-test.sqlite3'
    )
    ActiveRecord::Migration[5.0].subclasses.each do |migrate|
      migrate.migrate(:up)
    end
  end

  it "has a version number" do
    expect(WeatherModel::VERSION).not_to be nil
  end

  describe 'test factory_girl' do
    before do
      @obj = create(:city_forecast)  
    end

    it 'Test factory_girl' do
      expect(@obj.class).to eq(Forecast::CityForecast), 'factory_girl 生成成功'
    end
    
  end

  it 'Forecast' do
    @expire = 1
    mysql = Forecast::CityForecast
    ohm = ohm
    ohm.expire @expire
    mysql_obj = mysql.create({
          datetime: '2017-05-16',
          cityname: 'shanghai',
          temp_high: '100',
          temp_low: '0',
          weather_text1: '酷热',
          weather_text1: '严寒',
          weather_pic: '酷热转严寒',
          win_dir: '东南西北风',
          win_speed: '1级',
        })
    expect(mysql.count).to eq(1)
    expect(ohm.all.count).to eq(1)
    expect(ohm.all.first.temp_low).to eq(mysql_obj.temp_low.to_s)
    expect(ohm.all.first.get_ttl).to eq(@expire)
    # 'ohm 对象 同步 mysql对象更新'
    mysql_obj.update(temp_low: 99)
    expect(ohm.all.first.temp_low).to eq('99'), 'ohm 对象 同步 mysql对象更新'
    sleep 1.5
    expect(ohm.all.count).to eq(0), 'TTL过期对象删除成功'
  end

  it 'aqi_forecast' do
    @aqi_forecast = Aqi::AqiForecast.create(
      datetime: Time.now,
      prompt: '',
      list: [
        {
          "period"=>"今天夜间（20时—06时）",
          "aqi"=>"35-55",
          "level"=>"优到良",
          "pripoll"=>"PM10"
        }, {
          "period"=>"明天上午（06时—12时）",
          "aqi"=>"35-55",
          "level"=>"优到良",
          "pripoll"=>"PM10"
        },
      ]
    )
    expect(Aqi::AqiForecast.all.first.list.class).to eq(Array), 'Mysql json_column 正常'
    expect(Aqi::AqiForecast::Ohm.all.first.list.class).to eq(Array), 'Ohm json_column 正常'
  end
  
  describe 'test Ohm # to_h' do
    before do
      @aqi_live = create(:aqi_live)
    end

    it 'test Ohm to_h' do
      ohm_obj = WeatherModel::Aqi::AqiLive::Ohm.all.first
      expect(ohm_obj.time_point.class).to eq(Time)
      expect(ohm_obj.aqi.class).to eq(Integer)
      expect(ohm_obj.co.class).to eq(Float)
      expect(ohm_obj.area.class).to eq(String)
      hash = ohm_obj.to_h
      expect(hash[:time_point].class).to eq(Time)
      expect(hash[:aqi].class).to eq(Integer)
      expect(hash[:co].class).to eq(Float)
      expect(hash[:area].class).to eq(String)
      
    end
  end
  
  it 'Test all source data' do
    [
      WeatherModel::Forecast::CityForecast,
      WeatherModel::AutoStation::AutoStationTenMins,
      WeatherModel::Aqi::AqiForecast,
      WeatherModel::Aqi::AqiLive,
    ].each { |source| 
      # create factory 
      source.all; source.const_get('Ohm').all.first; 
    }
  end
  
  after do
    ActiveRecord::Migration[5.0].subclasses.each do |migrate|
      migrate.migrate(:down) 
    end 
  end

end
